@@ -2441,7 +2441,7 @@ TEST_F(HooksDhcpv6SrvTest, host6Identifier_hwaddr) {
 // The callout libraries write their library index number to a marker
 // file upon load and unload, making it simple to test whether or not
 // the load and unload callouts have been invoked.
-TEST_F(LoadUnloadDhcpv6SrvTest, unloadLibaries) {
+TEST_F(LoadUnloadDhcpv6SrvTest, unloadLibraries) {
 
     ASSERT_NO_THROW(server_.reset(new NakedDhcpv6Srv(0)));
 
