@@ -1,22 +1,11 @@
-// Copyright (C) 2011-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2017 Internet Systems Consortium, Inc. ("ISC")
 //
-// Permission to use, copy, modify, and/or distribute this software for any
-// purpose with or without fee is hereby granted, provided that the above
-// copyright notice and this permission notice appear in all copies.
-//
-// THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
-// REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
-// AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
-// INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-// LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
-// OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-// PERFORMANCE OF THIS SOFTWARE.
+// This Source Code Form is subject to the terms of the Mozilla Public
+// License, v. 2.0. If a copy of the MPL was not distributed with this
+// file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
-
-// This must be included before udp_endpoint.h
-#include <asio.hpp>
-
+#include <asiolink/asio_wrapper.h>
 #include <asiolink/io_error.h>
 #include <asiolink/udp_endpoint.h>
 #include <dhcp/dhcp4.h>
@@ -29,6 +18,7 @@
 #include <util/io/pktinfo_utilities.h>
 
 #include <boost/foreach.hpp>
+#include <boost/scoped_ptr.hpp>
 
 #include <cstring>
 #include <errno.h>
@@ -50,7 +40,12 @@ namespace dhcp {
 
 IfaceMgr&
 IfaceMgr::instance() {
-    static IfaceMgr iface_mgr;
+    return (*instancePtr());
+}
+
+const IfaceMgrPtr&
+IfaceMgr::instancePtr() {
+    static IfaceMgrPtr iface_mgr(new IfaceMgr());
     return (iface_mgr);
 }
 
@@ -73,7 +68,7 @@ Iface::closeSockets() {
 
 void
 Iface::closeSockets(const uint16_t family) {
-    // Check that the correect 'family' value has been specified.
+    // Check that the correct 'family' value has been specified.
     // The possible values are AF_INET or AF_INET6. Note that, in
     // the current code they are used to differentiate that the
     // socket is used to transmit IPv4 or IPv6 traffic. However,
@@ -437,7 +432,7 @@ void IfaceMgr::stubDetectIfaces() {
     iface->flag_running_ = true;
 
     // Note that we claim that this is not a loopback. iface_mgr tries to open a
-    // socket on all interaces that are up, running and not loopback. As this is
+    // socket on all interfaces that are up, running and not loopback. As this is
     // the only interface we were able to detect, let's pretend this is a normal
     // interface.
     iface->flag_loopback_ = false;
@@ -795,10 +790,10 @@ IfaceMgr::getLocalAddress(const IOAddress& remote_addr, const uint16_t port) {
     }
 
     // Create socket that will be used to connect to remote endpoint.
-    asio::io_service io_service;
-    asio::ip::udp::socket sock(io_service);
+    boost::asio::io_service io_service;
+    boost::asio::ip::udp::socket sock(io_service);
 
-    asio::error_code err_code;
+    boost::system::error_code err_code;
     // If remote address is broadcast address we have to
     // allow this on the socket.
     if (remote_addr.isV4() &&
@@ -810,13 +805,17 @@ IfaceMgr::getLocalAddress(const IOAddress& remote_addr, const uint16_t port) {
         // @todo: We don't specify interface in any way here. 255.255.255.255
         // We can very easily end up with a socket working on a different
         // interface.
-        sock.open(asio::ip::udp::v4(), err_code);
+
+        // zero out the errno to be safe
+        errno = 0;
+
+        sock.open(boost::asio::ip::udp::v4(), err_code);
         if (err_code) {
             const char* errstr = strerror(errno);
             isc_throw(Unexpected, "failed to open UDPv4 socket, reason:"
                       << errstr);
         }
-        sock.set_option(asio::socket_base::broadcast(true), err_code);
+        sock.set_option(boost::asio::socket_base::broadcast(true), err_code);
         if (err_code) {
             sock.close();
             isc_throw(Unexpected, "failed to enable broadcast on the socket");
@@ -831,9 +830,9 @@ IfaceMgr::getLocalAddress(const IOAddress& remote_addr, const uint16_t port) {
     }
 
     // Once we are connected socket object holds local endpoint.
-    asio::ip::udp::socket::endpoint_type local_endpoint =
+    boost::asio::ip::udp::socket::endpoint_type local_endpoint =
         sock.local_endpoint();
-    asio::ip::address local_address(local_endpoint.address());
+    boost::asio::ip::address local_address(local_endpoint.address());
 
     // Close the socket.
     sock.close();
@@ -881,14 +880,13 @@ IfaceMgr::send(const Pkt4Ptr& pkt) {
 }
 
 
-boost::shared_ptr<Pkt4>
-IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
+Pkt4Ptr IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
     // Sanity check for microsecond timeout.
     if (timeout_usec >= 1000000) {
         isc_throw(BadValue, "fractional timeout must be shorter than"
                   " one million microseconds");
     }
-    const SocketInfo* candidate = 0;
+    boost::scoped_ptr<SocketInfo> candidate;
     IfacePtr iface;
     fd_set sockets;
     int maxfd = 0;
@@ -927,6 +925,9 @@ IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
     select_timeout.tv_sec = timeout_sec;
     select_timeout.tv_usec = timeout_usec;
 
+    // zero out the errno to be safe
+    errno = 0;
+
     int result = select(maxfd + 1, &sockets, NULL, NULL, &select_timeout);
 
     if (result == 0) {
@@ -936,7 +937,7 @@ IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
     } else if (result < 0) {
         // In most cases we would like to know whether select() returned
         // an error because of a signal being received  or for some other
-        // reasaon. This is because DHCP servers use signals to trigger
+        // reason. This is because DHCP servers use signals to trigger
         // certain actions, like reconfiguration or graceful shutdown.
         // By catching a dedicated exception the caller will know if the
         // error returned by the function is due to the reception of the
@@ -970,7 +971,7 @@ IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
     BOOST_FOREACH(iface, ifaces_) {
         BOOST_FOREACH(SocketInfo s, iface->getSockets()) {
             if (FD_ISSET(s.sockfd_, &sockets)) {
-                candidate = &(s);
+                candidate.reset(new SocketInfo(s));
                 break;
             }
         }
@@ -995,7 +996,7 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
                   " one million microseconds");
     }
 
-    const SocketInfo* candidate = 0;
+    boost::scoped_ptr<SocketInfo> candidate;
     fd_set sockets;
     int maxfd = 0;
 
@@ -1034,6 +1035,9 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     select_timeout.tv_sec = timeout_sec;
     select_timeout.tv_usec = timeout_usec;
 
+    // zero out the errno to be safe
+    errno = 0;
+
     int result = select(maxfd + 1, &sockets, NULL, NULL, &select_timeout);
 
     if (result == 0) {
@@ -1043,7 +1047,7 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     } else if (result < 0) {
         // In most cases we would like to know whether select() returned
         // an error because of a signal being received  or for some other
-        // reasaon. This is because DHCP servers use signals to trigger
+        // reason. This is because DHCP servers use signals to trigger
         // certain actions, like reconfiguration or graceful shutdown.
         // By catching a dedicated exception the caller will know if the
         // error returned by the function is due to the reception of the
@@ -1077,7 +1081,7 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     BOOST_FOREACH(IfacePtr iface, ifaces_) {
         BOOST_FOREACH(SocketInfo s, iface->getSockets()) {
             if (FD_ISSET(s.sockfd_, &sockets)) {
-                candidate = &(s);
+                candidate.reset(new SocketInfo(s));
                 break;
             }
         }
@@ -1157,7 +1161,7 @@ IfaceMgr::getSocket(isc::dhcp::Pkt4 const& pkt) {
     }
 
     const Iface::SocketCollection& socket_collection = iface->getSockets();
-    // A candidate being an end of the iterator marks that it is a begining of
+    // A candidate being an end of the iterator marks that it is a beginning of
     // the socket search and that the candidate needs to be set to the first
     // socket found.
     Iface::SocketCollection::const_iterator candidate = socket_collection.end();
