@@ -146,7 +146,7 @@ TEST_F(MasterLoaderTest, basicLoad) {
     setLoader(TEST_DATA_SRCDIR "/example.org", Name("example.org."),
               RRClass::IN(), MasterLoader::MANY_ERRORS);
 
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
 
     // The following three should be set to 0 initially in case the loader
     // is constructed from a file name.
@@ -154,7 +154,7 @@ TEST_F(MasterLoaderTest, basicLoad) {
     EXPECT_EQ(0, loader_->getPosition());
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
 
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
@@ -191,7 +191,7 @@ TEST_F(MasterLoaderTest, include) {
                   MasterLoader::MANY_ERRORS);
 
         loader_->load();
-        EXPECT_TRUE(loader_->loadedSucessfully());
+        EXPECT_TRUE(loader_->loadedSuccessfully());
         EXPECT_TRUE(errors_.empty());
         EXPECT_TRUE(warnings_.empty());
 
@@ -286,7 +286,7 @@ TEST_F(MasterLoaderTest, origin) {
                   MasterLoader::MANY_ERRORS);
 
         loader_->load();
-        EXPECT_TRUE(loader_->loadedSucessfully());
+        EXPECT_TRUE(loader_->loadedSuccessfully());
         EXPECT_TRUE(errors_.empty());
         // There's a relative origin in it, we warn about that.
         EXPECT_EQ(1, warnings_.size());
@@ -326,7 +326,7 @@ TEST_F(MasterLoaderTest, generate) {
                   MasterLoader::MANY_ERRORS);
 
         loader_->load();
-        EXPECT_TRUE(loader_->loadedSucessfully());
+        EXPECT_TRUE(loader_->loadedSuccessfully());
         EXPECT_TRUE(errors_.empty());
 
         // The "before" and "after" scaffolding below checks that no
@@ -349,7 +349,7 @@ TEST_F(MasterLoaderTest, generateRelativeLHS) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("example.org", RRType::NS(), "ns1.example.org.");
@@ -366,7 +366,7 @@ TEST_F(MasterLoaderTest, generateInFront) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("9host.example.org", RRType::TXT(), "9 pomegranate");
@@ -383,7 +383,7 @@ TEST_F(MasterLoaderTest, generateInMiddle) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("num9-host.example.org", RRType::TXT(), "This is 9 pomegranate");
@@ -400,7 +400,7 @@ TEST_F(MasterLoaderTest, generateAtEnd) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("num9-host.example.org", RRType::TXT(), "Pomegranate9");
@@ -416,7 +416,7 @@ TEST_F(MasterLoaderTest, generateStripsQuotes) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("example.org", RRType::MX(), "1 mx1.example.org.");
@@ -432,7 +432,7 @@ TEST_F(MasterLoaderTest, generateWithDoublePlaceholder) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("host9.example.org", RRType::TXT(), "This is $ pomegranate");
@@ -448,7 +448,7 @@ TEST_F(MasterLoaderTest, generateWithEscape) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("host9.example.org", RRType::TXT(), "This is \\$\\pomegranate");
@@ -469,7 +469,7 @@ TEST_F(MasterLoaderTest, generateWithParams) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("host2.example.org", RRType::A(), "192.0.2.2");
@@ -499,7 +499,7 @@ TEST_F(MasterLoaderTest, generateWithStep) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("host2.example.org", RRType::A(), "192.0.2.2");
@@ -558,7 +558,7 @@ TEST_F(MasterLoaderTest, generateWithModifiers) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
 
     checkRR("host3.example.org", RRType::A(), "192.0.2.1");
@@ -609,7 +609,7 @@ TEST_F(MasterLoaderTest, generateWithNoModifiers) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     ASSERT_EQ(2, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -629,7 +629,7 @@ TEST_F(MasterLoaderTest, generateWithBadModifiers) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     ASSERT_EQ(2, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -648,7 +648,7 @@ TEST_F(MasterLoaderTest, generateMissingRange) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -665,7 +665,7 @@ TEST_F(MasterLoaderTest, generateMissingLHS) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -682,7 +682,7 @@ TEST_F(MasterLoaderTest, generateMissingType) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -699,7 +699,7 @@ TEST_F(MasterLoaderTest, generateMissingRHS) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -716,7 +716,7 @@ TEST_F(MasterLoaderTest, generateWithBadRangeSyntax) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -734,7 +734,7 @@ TEST_F(MasterLoaderTest, generateWithInvalidRange) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -751,7 +751,7 @@ TEST_F(MasterLoaderTest, generateWithInvalidClass) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -768,7 +768,7 @@ TEST_F(MasterLoaderTest, generateWithNoAvailableTTL) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken GENERATE
     EXPECT_TRUE(warnings_.empty());
 
@@ -787,7 +787,7 @@ TEST_F(MasterLoaderTest, popAfterError) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size()); // For the broken RR
     EXPECT_EQ(1, warnings_.size()); // For missing EOLN
 
@@ -803,7 +803,7 @@ TEST_F(MasterLoaderTest, streamConstructor) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
 
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
 
     // Unlike the basicLoad test, if we construct the loader from a stream
     // getSize() returns the data size in the stream immediately after the
@@ -812,7 +812,7 @@ TEST_F(MasterLoaderTest, streamConstructor) {
     EXPECT_EQ(0, loader_->getPosition());
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
 
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
@@ -831,9 +831,9 @@ TEST_F(MasterLoaderTest, incrementalLoad) {
     setLoader(TEST_DATA_SRCDIR "/example.org", Name("example.org."),
               RRClass::IN(), MasterLoader::MANY_ERRORS);
 
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_FALSE(loader_->loadIncremental(2));
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
 
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
@@ -848,7 +848,7 @@ TEST_F(MasterLoaderTest, incrementalLoad) {
 
     // Load the rest.
     EXPECT_TRUE(loader_->loadIncremental(20));
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
 
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
@@ -977,9 +977,9 @@ TEST_F(MasterLoaderTest, brokenZone) {
             stringstream zone_stream(zone);
             setLoader(zone_stream, Name("example.org."), RRClass::IN(),
                       MasterLoader::DEFAULT);
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_THROW(loader_->load(), MasterLoaderError);
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_EQ(1, errors_.size());
             if (ec->reason != NULL) {
                 checkCallbackMessage(errors_.at(0), ec->reason, 2);
@@ -999,9 +999,9 @@ TEST_F(MasterLoaderTest, brokenZone) {
             stringstream zone_stream(zone);
             setLoader(zone_stream, Name("example.org."), RRClass::IN(),
                       MasterLoader::MANY_ERRORS);
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_NO_THROW(loader_->load());
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_EQ(1, errors_.size());
             EXPECT_TRUE(warnings_.empty());
             checkRR("example.org", RRType::SOA(), "ns1.example.org. "
@@ -1019,9 +1019,9 @@ TEST_F(MasterLoaderTest, brokenZone) {
             stringstream zone_stream(zoneEOF);
             setLoader(zone_stream, Name("example.org."), RRClass::IN(),
                       MasterLoader::MANY_ERRORS);
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_NO_THROW(loader_->load());
-            EXPECT_FALSE(loader_->loadedSucessfully());
+            EXPECT_FALSE(loader_->loadedSuccessfully());
             EXPECT_EQ(1, errors_.size()) << errors_[0] << "\n" << errors_[1];
             // The unexpected EOF warning
             EXPECT_EQ(1, warnings_.size());
@@ -1045,7 +1045,7 @@ TEST_F(MasterLoaderTest, includeWithGarbage) {
               MasterLoader::MANY_ERRORS);
 
     EXPECT_NO_THROW(loader_->load());
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     ASSERT_EQ(1, errors_.size());
     checkCallbackMessage(errors_.at(0), "Extra tokens at the end of line", 1);
     // It says something about extra tokens at the end
@@ -1064,7 +1064,7 @@ TEST_F(MasterLoaderTest, originWithGarbage) {
     setLoader(ss, Name("example.org."), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     EXPECT_NO_THROW(loader_->load());
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     ASSERT_EQ(1, errors_.size());
     checkCallbackMessage(errors_.at(0), "Extra tokens at the end of line", 1);
     EXPECT_TRUE(warnings_.empty());
@@ -1086,7 +1086,7 @@ TEST_F(MasterLoaderTest, includeAndOrigin) {
               MasterLoader::MANY_ERRORS);
     // Successfully load the data
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
     // And check it's the correct data
@@ -1106,7 +1106,7 @@ TEST_F(MasterLoaderTest, includeAndBadOrigin) {
     setLoader(ss, Name("example.org"), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size());
     checkCallbackMessage(errors_.at(0), "duplicate period in example..org.",
                          1);
@@ -1125,7 +1125,7 @@ TEST_F(MasterLoaderTest, includeOriginRestore) {
               MasterLoader::MANY_ERRORS);
     // Successfully load the data
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
     // And check it's the correct data
@@ -1144,7 +1144,7 @@ TEST_F(MasterLoaderTest, includeAndInitialWS) {
               MasterLoader::MANY_ERRORS);
     // Successfully load the data
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     EXPECT_EQ(1, warnings_.size());
     checkCallbackMessage(warnings_.at(0),
@@ -1173,7 +1173,7 @@ TEST_F(MasterLoaderTest, ttlDirective) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     checkRR("example.org", RRType::A(), "192.0.2.1", RRTTL(1800));
     checkRR("a.example.org", RRType::A(), "192.0.2.2", RRTTL(100));
     checkRR("b.example.org", RRType::A(), "192.0.2.3", RRTTL(3600));
@@ -1190,7 +1190,7 @@ TEST_F(MasterLoaderTest, ttlFromSOA) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     checkRR("example.org", RRType::SOA(), ". . 0 0 0 0 1800", RRTTL(1800));
     checkRR("a.example.org", RRType::A(), "192.0.2.1", RRTTL(1800));
 
@@ -1209,7 +1209,7 @@ TEST_F(MasterLoaderTest, ttlFromPrevious) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     checkRR("a.example.org", RRType::A(), "192.0.2.1", RRTTL(1800));
     checkRR("b.example.org", RRType::A(), "192.0.2.2", RRTTL(1800));
     checkRR("c.example.org", RRType::A(), "192.0.2.3", RRTTL(1800));
@@ -1237,7 +1237,7 @@ TEST_F(MasterLoaderTest, RRParamsOrdering) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     checkRR("a.example.org", RRType::A(), "192.0.2.1", RRTTL(1800));
     checkRR("b.example.org", RRType::A(), "192.0.2.2", RRTTL(1800));
     checkRR("c.example.org", RRType::A(), "192.0.2.3", RRTTL(3600));
@@ -1257,7 +1257,7 @@ TEST_F(MasterLoaderTest, ttlFromPreviousSOA) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::DEFAULT);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
 
     checkRR("example.org", RRType::SOA(), ". . 0 0 0 0 1800", RRTTL(100));
     checkRR("a.example.org", RRType::A(), "192.0.2.1", RRTTL(100));
@@ -1280,7 +1280,7 @@ TEST_F(MasterLoaderTest, ttlUnknownAndContinue) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     checkRR("b.example.org", RRType::A(), "192.0.2.2", RRTTL(1800));
 
     EXPECT_TRUE(warnings_.empty());
@@ -1295,7 +1295,7 @@ TEST_F(MasterLoaderTest, ttlUnknownAndEOF) {
     setLoader(zone_stream, Name("example.org."), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_TRUE(rrsets_.empty());
 
     EXPECT_EQ(1, errors_.size());
@@ -1319,7 +1319,7 @@ TEST_F(MasterLoaderTest, ttlOverflow) {
               MasterLoader::DEFAULT);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_EQ(3, rrsets_.size());
 
     checkRR("example.org", RRType::SOA(), ". . 0 0 0 0 2147483648", RRTTL(0));
@@ -1380,7 +1380,7 @@ TEST_F(MasterLoaderTest, noEOLN) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     // There should be one warning about the EOLN
     EXPECT_EQ(1, warnings_.size());
@@ -1396,7 +1396,7 @@ TEST_F(MasterLoaderTest, noPreviousName) {
     setLoader(ss, Name("example.org."), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     loader_->load();
-    EXPECT_FALSE(loader_->loadedSucessfully());
+    EXPECT_FALSE(loader_->loadedSuccessfully());
     EXPECT_EQ(1, errors_.size());
     checkCallbackMessage(errors_.at(0), "No previous name to use in place of "
                          "initial whitespace", 1);
@@ -1411,7 +1411,7 @@ TEST_F(MasterLoaderTest, previousInInclude) {
     setLoader(ss, Name("example.org"), RRClass::IN(),
               MasterLoader::MANY_ERRORS);
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     // There should be one warning about the EOLN
     EXPECT_EQ(1, warnings_.size());
@@ -1429,7 +1429,7 @@ TEST_F(MasterLoaderTest, numericOwnerName) {
               MasterLoader::MANY_ERRORS);
 
     loader_->load();
-    EXPECT_TRUE(loader_->loadedSucessfully());
+    EXPECT_TRUE(loader_->loadedSuccessfully());
     EXPECT_TRUE(errors_.empty());
     EXPECT_TRUE(warnings_.empty());
 
