@@ -446,6 +446,7 @@ DControllerBase::configWriteHandler(const std::string&,
         }
     }
 
+
     // Ok, it's time to write the file.
     size_t size = 0;
     ElementPtr cfg = process_->getCfgMgr()->getContext()->toElement();
@@ -471,7 +472,7 @@ DControllerBase::configWriteHandler(const std::string&,
     }
 
     try {
-        size = writeConfigFile(filename);
+        size = writeConfigFile(filename, cfg);
     } catch (const isc::Exception& ex) {
         return (createAnswer(COMMAND_ERROR,
                              std::string("Error during write-config:")
